@@ -37,6 +37,7 @@ Copyright (c) Intel Corporation (2009-2017).
 #include "llvm/Support/Casting.h"
 #include "llvm/Support/Path.h"
 #include "llvm/Support/Threading.h"
+#include "llvm/Support/TargetSelect.h"
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Support/Mutex.h"
 #include "llvm/Support/VirtualFileSystem.h"
@@ -80,7 +81,11 @@ Copyright (c) Intel Corporation (2009-2017).
 
 using namespace Intel::OpenCL::ClangFE;
 
-llvm::ManagedStatic<llvm::sys::SmartMutex<true>> compileMutex;
+static volatile bool lazyCCInit =
+    true; // the flag must be 'volatile' to prevent caching in a CPU register
+static llvm::sys::Mutex lazyCCInitMutex;
+
+static llvm::ManagedStatic<llvm::sys::SmartMutex<true> > compileMutex;
 
 void CommonClangTerminate() { llvm::llvm_shutdown(); }
 
@@ -88,40 +93,37 @@ void CommonClangTerminate() { llvm::llvm_shutdown(); }
 // from a DllMain function (Windows specific), or from a function
 // w\ __attribute__ ((constructor)) (Linux specific).
 void CommonClangInitialize() {
-  // CommonClangTerminate calls llvm_shutdown to deallocate resources used
-  // by LLVM libraries. llvm_shutdown uses static mutex to make it safe for
-  // multi-threaded envirounment and LLVM libraries user is expected call
-  // llvm_shutdown before static object are destroyed, so we use atexit to
-  // satisfy this requirement.
-  llvm::once_flag OnceFlag;
-  llvm::call_once(OnceFlag, []() { atexit(CommonClangTerminate); });
+  if (lazyCCInit) {
+    llvm::sys::ScopedLock lock(lazyCCInitMutex);
+
+    if (lazyCCInit) {
+      // CommonClangTerminate calls llvm_shutdown to deallocate resources used
+      // by LLVM libraries. llvm_shutdown uses static mutex to make it safe for
+      // multi-threaded envirounment and LLVM libraries user is expected call
+      // llvm_shutdown before static object are destroyed, so we use atexit to
+      // satisfy this requirement.
+      atexit(CommonClangTerminate);
+      llvm::InitializeAllTargets();
+      llvm::InitializeAllAsmPrinters();
+      llvm::InitializeAllAsmParsers();
+      llvm::InitializeAllTargetMCs();
+      lazyCCInit = false;
+    }
+  }
 }
 
 static bool GetHeaders(std::vector<Resource> &Result) {
-  struct {
-    const char *ID;
-    const char *Name;
-  } Headers[] = {{OPENCL_C_H, "opencl-c.h"},
-                 {OPENCL_C_BASE_H, "opencl-c-base.h"},
-                 {OPENCL_C_12_SPIR_PCM, "opencl-c-12-spir.pcm"},
-                 {OPENCL_C_20_SPIR_PCM, "opencl-c-20-spir.pcm"},
-                 {OPENCL_C_30_SPIR_PCM, "opencl-c-30-spir.pcm"},
-                 {OPENCL_C_12_SPIR64_PCM, "opencl-c-12-spir64.pcm"},
-                 {OPENCL_C_20_SPIR64_PCM, "opencl-c-20-spir64.pcm"},
-                 {OPENCL_C_30_SPIR64_PCM, "opencl-c-30-spir64.pcm"},
-                 {OPENCL_C_12_SPIR_FP64_PCM, "opencl-c-12-spir-fp64.pcm"},
-                 {OPENCL_C_20_SPIR_FP64_PCM, "opencl-c-20-spir-fp64.pcm"},
-                 {OPENCL_C_30_SPIR_FP64_PCM, "opencl-c-30-spir-fp64.pcm"},
-                 {OPENCL_C_12_SPIR64_FP64_PCM, "opencl-c-12-spir64-fp64.pcm"},
-                 {OPENCL_C_20_SPIR64_FP64_PCM, "opencl-c-20-spir64-fp64.pcm"},
-                 {OPENCL_C_30_SPIR64_FP64_PCM, "opencl-c-30-spir64-fp64.pcm"},
-                 {OPENCL_C_MODULE_MAP, "module.modulemap"}};
+  struct {const char *ID; const char *Name;} Headers[] = {
+    {OPENCL_C_H,             "opencl-c.h"},
+    {OPENCL_C_BASE_H,        "opencl-c-base.h"},
+  };
 
   Result.clear();
   Result.reserve(sizeof(Headers) / sizeof(*Headers));
 
   ResourceManager &RM = ResourceManager::instance();
-  for (auto Header : Headers) {
+
+  for (auto Header:Headers) {
     Resource R = RM.get_resource(Header.Name, Header.ID, "PCM", true);
     if (!R) {
       assert(0 && "Resource not found");
@@ -197,9 +199,6 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
   CommonClangInitialize();
 
   try {
-#ifdef _WIN32
-    llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
-#endif
     std::unique_ptr<OCLFEBinaryResult> pResult(new OCLFEBinaryResult());
 
     // Create the clang compiler
@@ -211,9 +210,8 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
     // Prepare error log
     llvm::raw_string_ostream err_ostream(pResult->getLogRef());
     {
-#ifndef _WIN32
-      llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
-#endif
+      llvm::sys::SmartScopedLock<true> compileGuard {*compileMutex};
+
       // Parse options
       optionsParser.processOptions(pszOptions, pszOptionsEx);
 
@@ -332,9 +330,7 @@ Compile(const char *pszProgramSource, const char **pInputHeaders,
       err_ostream.flush();
     }
     {
-#ifndef _WIN32
-      llvm::sys::SmartScopedLock<true> compileGuard{*compileMutex};
-#endif
+      llvm::sys::SmartScopedLock<true> compileGuard {*compileMutex};
       if (pBinaryResult) {
         *pBinaryResult = pResult.release();
       }
