@@ -21,13 +21,12 @@ Copyright (c) Intel Corporation (2009-2017).
 #ifndef COMMON_CLANG_OPTIONS_H
 #define COMMON_CLANG_OPTIONS_H
 
-#include "LLVMSPIRVOpts.h"
-#include "clang/Basic/OpenCLOptions.h"
+#include "llvm/Option/Arg.h"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/StringRef.h"
-#include "llvm/Option/Arg.h"
 #include "llvm/Option/ArgList.h"
 #include "llvm/Option/Option.h"
+#include "clang/Basic/OpenCLOptions.h"
 
 #include <list>
 
@@ -150,7 +149,7 @@ public:
   // Validates and prepares the effective options to pass to clang upon
   // compilation
   //
-  int processOptions(const char *pszOptions, const char *pszOptionsEx);
+  void processOptions(const char *pszOptions, const char *pszOptionsEx);
 
   //
   // Just validates the user supplied OpenCL compile options
@@ -176,13 +175,6 @@ public:
   std::string getEffectiveOptionsAsString() const;
 
   bool hasEmitSPIRV() const { return m_emitSPIRV; }
-
-  bool hasSPIRVExt() const { return m_hasSPIRVExt; }
-
-  SPIRV::TranslatorOpts::ExtensionsStatusMap getSPIRVExtStatusMap() const {
-    return m_SPIRVExtStatusMap;
-  }
-
   bool hasOptDisable() const { return m_optDisable; }
 
 private:
@@ -192,8 +184,6 @@ private:
   llvm::SmallVector<const char *, 16> m_effectiveArgsRaw;
   std::string m_sourceName;
   bool m_emitSPIRV;
-  bool m_hasSPIRVExt = false;
-  SPIRV::TranslatorOpts::ExtensionsStatusMap m_SPIRVExtStatusMap = {};
   bool m_optDisable;
 };
 
